@@ -33,6 +33,7 @@
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomeui/gnome-propertybox.h>
 #include <libgnomeui/gnome-pixmap.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
 #include <gst/gst.h>
 #include "typedefs.h"
@@ -61,13 +62,18 @@ typedef struct {
 	GtkWidget   *dialog;
 	GtkWidget   *e_alltrack_radiobutton;
 	GtkWidget   *e_selected_radiobutton;
+#ifdef HAVE_GTK_2_5
+	GtkWidget   *e_destination_filechooserbutton;
+#else
 	GtkWidget   *e_destination_entry;
 	GtkWidget   *e_destination_button;
+#endif
 	GtkWidget   *e_ogg_radiobutton;
 	GtkWidget   *e_flac_radiobutton;
 	GtkWidget   *e_mp3_radiobutton;
 	GtkWidget   *e_wave_radiobutton;
 	GtkWidget   *e_save_playlist_checkbutton;
+	GtkWidget   *e_options_expander;
 } DialogData;
 
 
@@ -135,6 +141,7 @@ dlg_extract__start_ripping (GtkWidget *dialog)
 		    file_format,
 		    goo_player_cd_get_album (data->player_cd),
 		    goo_player_cd_get_artist (data->player_cd),
+		    goo_player_get_year (GOO_PLAYER (data->player_cd)),
 		    goo_player_cd_get_genre (data->player_cd),
 		    g_list_length (data->songs),
 		    tracks_to_rip);
@@ -154,12 +161,22 @@ ok_cb (GtkWidget  *widget,
        DialogData *data)
 {
 	const char    *destination;
+#ifdef HAVE_GTK_2_5
+	char          *unesc_destination = NULL;
+#endif
 	GooFileFormat  file_format = GOO_FILE_FORMAT_OGG;
 
 	/* save preferences */
 
+#ifdef HAVE_GTK_2_5
+	destination = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->e_destination_filechooserbutton));
+	unesc_destination = gnome_vfs_unescape_string (destination, "");
+	eel_gconf_set_path (PREF_EXTRACT_DESTINATION, unesc_destination);
+	g_free (unesc_destination);
+#else
 	destination = gtk_entry_get_text (GTK_ENTRY (data->e_destination_entry));
 	eel_gconf_set_path (PREF_EXTRACT_DESTINATION, destination);
+#endif
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_ogg_radiobutton)))
 		file_format = GOO_FILE_FORMAT_OGG;
@@ -214,6 +231,9 @@ help_cb (GtkWidget  *widget,
 }
 
 
+#ifndef HAVE_GTK_2_5
+
+
 static void
 open_response_cb (GtkDialog  *file_sel,
 		  int         button_number,
@@ -222,8 +242,12 @@ open_response_cb (GtkDialog  *file_sel,
 	DialogData *data = (DialogData*) userdata;
 
 	if (button_number == GTK_RESPONSE_ACCEPT) {
-		_gtk_entry_set_filename_text (GTK_ENTRY (data->e_destination_entry),
-					      gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel)));
+		char *unesc_destination = NULL;
+		
+		unesc_destination = gnome_vfs_unescape_string (gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (file_sel)), "");
+		
+		_gtk_entry_set_filename_text (GTK_ENTRY (data->e_destination_entry), unesc_destination);
+		g_free (unesc_destination);
 	}
 	gtk_widget_destroy (GTK_WIDGET (file_sel));
 }
@@ -234,7 +258,8 @@ destination_button_clicked_cb (GtkWidget  *button,
 			       DialogData *data)
 {
 	GtkWidget *file_sel;
-	
+	char      *esc_uri;
+
 	file_sel = gtk_file_chooser_dialog_new (_("Choose destination folder"),
 						GTK_WINDOW (data->dialog),
 						GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, 
@@ -242,8 +267,10 @@ destination_button_clicked_cb (GtkWidget  *button,
 						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 						NULL);
 	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), TRUE);
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel), gtk_entry_get_text (GTK_ENTRY (data->e_destination_entry)));
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), FALSE);
+	esc_uri = gnome_vfs_escape_host_and_path_string (gtk_entry_get_text (GTK_ENTRY (data->e_destination_entry)));
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (file_sel), esc_uri);
+	g_free (esc_uri);
 
 	g_signal_connect (G_OBJECT (file_sel),
 			  "response",
@@ -259,6 +286,9 @@ destination_button_clicked_cb (GtkWidget  *button,
 }
 
 
+#endif
+
+
 
 
 
@@ -277,6 +307,9 @@ dlg_extract (GooWindow *window)
 	GstElement       *encoder;
 	gboolean          ogg_encoder, flac_encoder, mp3_encoder, wave_encoder;
 	gboolean          find_first_available;
+#ifdef HAVE_GTK_2_5
+	char             *esc_uri = NULL;
+#endif
 
 	goo_window_stop (window);
 
@@ -300,10 +333,16 @@ dlg_extract (GooWindow *window)
 
 	data->dialog = glade_xml_get_widget (data->gui, "extract_dialog");
 
+	data->e_options_expander = glade_xml_get_widget (data->gui, "e_options_expander");
+
 	data->e_alltrack_radiobutton = glade_xml_get_widget (data->gui, "e_alltrack_radiobutton");
 	data->e_selected_radiobutton = glade_xml_get_widget (data->gui, "e_selected_radiobutton");
+#ifdef HAVE_GTK_2_5
+	data->e_destination_filechooserbutton = glade_xml_get_widget (data->gui, "e_destination_filechooserbutton");
+#else
 	data->e_destination_entry = glade_xml_get_widget (data->gui, "e_destination_entry");
 	data->e_destination_button = glade_xml_get_widget (data->gui, "e_destination_button");
+#endif
 	data->e_ogg_radiobutton = glade_xml_get_widget (data->gui, "e_ogg_radiobutton");
 	data->e_flac_radiobutton = glade_xml_get_widget (data->gui, "e_flac_radiobutton");
 	data->e_mp3_radiobutton = glade_xml_get_widget (data->gui, "e_mp3_radiobutton");
@@ -321,6 +360,12 @@ dlg_extract (GooWindow *window)
 
 	g_object_set_data (G_OBJECT (data->dialog), "dialog_data", data);
 
+	if (eel_gconf_get_boolean (PREF_EXTRACT_FIRST_TIME, TRUE)) {
+		eel_gconf_set_boolean (PREF_EXTRACT_FIRST_TIME, FALSE);
+		gtk_expander_set_expanded (GTK_EXPANDER (data->e_options_expander), TRUE);
+	} else
+		gtk_expander_set_expanded (GTK_EXPANDER (data->e_options_expander), FALSE);
+
 	selected = g_list_length (data->selected_songs);
 	if (selected == 0) {
 		gtk_widget_set_sensitive (data->e_selected_radiobutton, FALSE);
@@ -331,7 +376,17 @@ dlg_extract (GooWindow *window)
 	}
 
 	path = eel_gconf_get_path (PREF_EXTRACT_DESTINATION, "~");
+
+#ifdef HAVE_GTK_2_5
+	gtk_widget_show (data->e_destination_filechooserbutton);
+	esc_uri = gnome_vfs_escape_host_and_path_string (path);
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->e_destination_filechooserbutton), esc_uri);
+	g_free (esc_uri);
+#else
+	gtk_widget_show (glade_xml_get_widget (data->gui, "e_destination_hbox"));
 	gtk_entry_set_text (GTK_ENTRY (data->e_destination_entry), path);
+#endif
+
 	g_free (path);
 
 	encoder = gst_element_factory_make (OGG_ENCODER, "encoder");
@@ -459,14 +514,15 @@ dlg_extract (GooWindow *window)
 			  "clicked",
 			  G_CALLBACK (ok_cb),
 			  data);
+#ifndef HAVE_GTK_2_5
 	g_signal_connect (G_OBJECT (data->e_destination_button), 
 			  "clicked",
 			  G_CALLBACK (destination_button_clicked_cb),
 			  data);
-
+#endif
 	/* run dialog. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
-	gtk_widget_show_all (data->dialog);
+	gtk_widget_show (data->dialog);
 }
