@@ -26,6 +26,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <math.h>
 
 #include <gtk/gtk.h>
 #include <libgnome/libgnome.h>
@@ -34,6 +35,7 @@
 #include <libgnomeui/gnome-propertybox.h>
 #include <libgnomeui/gnome-pixmap.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
 #include <gst/gst.h>
 #include "typedefs.h"
@@ -56,7 +58,7 @@
 #define UPDATE_DELAY 400
 #define DEFAULT_OGG_QUALITY 0.3
 #define DEFAULT_FLAC_COMPRESSION 5
-#define DEFAULT_MP3_BITRATE 128
+#define DEFAULT_MP3_QUALITY 4
 #define BUFFER_SIZE 1024
 
 typedef struct {
@@ -88,6 +90,7 @@ typedef struct {
 	int            prev_tracks_sectors;
 	char          *current_file;
 	gboolean       ripping;
+	GTimer        *timer;
 
 	GladeXML      *gui;
 
@@ -98,6 +101,20 @@ typedef struct {
 } DialogData;
 
 
+/* From lame.h */
+typedef enum vbr_mode_e {
+  vbr_off=0,
+  vbr_mt,               /* obsolete, same as vbr_mtrh */
+  vbr_rh,
+  vbr_abr,
+  vbr_mtrh,
+  vbr_max_indicator,    /* Don't use this! It's used for sanity checks.       */  
+  vbr_default=vbr_rh    /* change this to change the default VBR mode of LAME */
+} vbr_mode;
+
+static int mp3_bitrate[] = { 320, 256, 224, 192, 160, 128, 112, 96, 80, 64 };
+
+
 /* called when the main dialog is closed. */
 static void
 destroy_cb (GtkWidget  *widget, 
@@ -116,6 +133,9 @@ destroy_cb (GtkWidget  *widget,
 		gst_object_unref (GST_OBJECT (data->rip_thread));
 	}
 
+	if (data->timer != NULL)
+		g_timer_destroy (data->timer);
+
 	goo_cdrom_unlock_tray (data->cdrom);
 	g_object_unref (data->cdrom);
 	
@@ -158,15 +178,46 @@ rip_next_track (gpointer callback_data)
 }
 
 
+static char *
+time_pretty_print (double s)
+{
+	char *r, *rr;
+	int   i;
+	
+	if (s < 0.0)
+		return g_strdup ("00:00");
+
+	i = (int) (floor (s + 0.5));
+	r = g_strdup_printf ("%2d:%2d", i / 60, i % 60);
+	for (rr = r; *rr != 0; rr++)
+		if (*rr == ' ')
+			*rr = '0';
+
+	return r;
+}
+
+
 static gboolean
 update_ui (gpointer callback_data)
 {
 	DialogData *data = callback_data;
 	double      fraction;
+	double      e, r;
+	char       *elapsed, *s_e, *s_r;
 
 	fraction = ((double) (data->current_track_sectors + data->prev_tracks_sectors)) / (double) data->total_sectors;
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->r_progress_progressbar), fraction);
 
+	e = g_timer_elapsed (data->timer, NULL);
+	r = (e / fraction) - e;
+	s_e = time_pretty_print (e);
+	s_r = time_pretty_print (r);
+	elapsed = g_strdup_printf (_("Elapsed: %s / Remaining: %s"), s_e, s_r);
+	gtk_progress_bar_set_text  (GTK_PROGRESS_BAR (data->r_progress_progressbar), elapsed);
+	g_free (s_e);
+	g_free (s_r);
+	g_free (elapsed);
+
 	return FALSE;
 }
 
@@ -214,9 +265,10 @@ update_progress_cb (gpointer callback_data)
 static void
 create_pipeline (DialogData *data)
 {
-	float ogg_quality;
-	int   flac_compression;
-	int   mp3_bitrate;
+	float       ogg_quality;
+	int         flac_compression;
+	int         mp3_quality;
+	GParamSpec *pspec;
 
 	data->rip_thread = gst_thread_new ("rip_thread");
 		
@@ -234,6 +286,7 @@ create_pipeline (DialogData *data)
 			      "quality", ogg_quality,
 			      NULL);
 		break;
+
 	case GOO_FILE_FORMAT_FLAC:
 		data->encoder = gst_element_factory_make (FLAC_ENCODER, "encoder");
 		data->ext = "flac";
@@ -242,14 +295,24 @@ create_pipeline (DialogData *data)
 			      "quality", flac_compression,
 			      NULL);
 		break;
+
 	case GOO_FILE_FORMAT_MP3:
 		data->encoder = gst_element_factory_make (MP3_ENCODER, "encoder");
 		data->ext = "mp3";
-		mp3_bitrate = eel_gconf_get_integer (PREF_ENCODER_MP3_BITRATE, DEFAULT_MP3_BITRATE);
-		g_object_set (data->encoder,
-			      "bitrate", mp3_bitrate,
-			      NULL);
+		mp3_quality = eel_gconf_get_integer (PREF_ENCODER_MP3_QUALITY, DEFAULT_MP3_QUALITY);
+		pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (data->encoder), "vbr_quality");
+		if (pspec != NULL) {
+			g_object_set (data->encoder,
+				      "vbr", vbr_mtrh,
+				      "vbr_quality", mp3_quality,
+				      NULL);
+			g_param_spec_unref (pspec);
+		} else 
+			g_object_set (data->encoder,
+				      "bitrate", mp3_bitrate[mp3_quality],
+				      NULL);
 		break;
+
 	case GOO_FILE_FORMAT_WAVE:
 		data->encoder = gst_element_factory_make (WAVE_ENCODER, "encoder");
 		data->ext = "wav";
@@ -271,28 +334,25 @@ create_pipeline (DialogData *data)
 }
 
 
-static char *
-zero_padded (int n)
-{
-        static char  s[1024];
-        char        *t;
-
-        sprintf (s, "%2d", n);
-        for (t = s; (t != NULL) && (*t != 0); t++)
-                if (*t == ' ')
-                        *t = '0';
-
-        return s;
-}
-
-
 static char*
 get_destination_folder (DialogData *data)
 {
-	return g_build_filename (data->destination,
-				 data->artist,
-				 data->album,
-				 NULL);
+	char *artist_filename;
+	char *album_filename;
+	char *result;
+
+	artist_filename = tracktitle_to_filename (data->artist);
+	album_filename = tracktitle_to_filename (data->album);
+	
+	result = g_build_filename (data->destination,
+				   artist_filename,
+				   album_filename,
+				   NULL);
+
+	g_free (artist_filename);
+	g_free (album_filename);
+
+	return result;
 }
 
 
@@ -309,29 +369,68 @@ done_dialog_response_cb (GtkDialog  *dialog,
 	    && eel_gconf_get_boolean (PREF_RIPPER_VIEW_DISTINATION, FALSE)) {
 		GError *error  = NULL;
 		char   *folder = get_destination_folder (data);
-		char   *url    = g_strconcat ("file://", folder, NULL);
+		char   *scheme = NULL;
+		char   *url    = NULL;
+		
+		scheme = gnome_vfs_get_uri_scheme (folder);
+		if ((scheme == NULL) || (scheme == ""))
+			url = gnome_vfs_get_uri_from_local_path (folder);
+		else
+			url = g_strdup (folder);
 
 		if (! gnome_url_show (url, &error))
 			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->window), _("Could not display the destination folder"), &error);
 
-		g_free (folder);
+		g_free (scheme);
 		g_free (url);
+		g_free (folder);
 	}
 
 	gtk_widget_destroy (data->dialog);
 }
 
 
+static char *
+zero_padded (int n)
+{
+        static char  s[1024];
+        char        *t;
+
+        sprintf (s, "%2d", n);
+        for (t = s; (t != NULL) && (*t != 0); t++)
+                if (*t == ' ')
+                        *t = '0';
+
+        return s;
+}
+
+
+static char *
+get_track_filename (TrackInfo  *track,
+		    const char *ext)
+{
+	char *filename, *track_filename;
+
+	filename = tracktitle_to_filename (track->title);
+	track_filename = g_strdup_printf ("%s - %s.%s", zero_padded (track->number + 1), filename, ext);
+	g_free (filename);
+
+	return track_filename;
+}
+
+
 static void
 save_playlist (DialogData *data)
 {
 	char           *folder;
-	char           *filename;
+	char           *filename, *album_filename;
 	GnomeVFSResult  result;
 	GnomeVFSHandle *handle;
 
 	folder = get_destination_folder (data);
-	filename = g_strconcat ("file://", folder, "/", data->album, ".pls", NULL);
+	album_filename = tracktitle_to_filename (data->album);
+	filename = g_strconcat ("file://", folder, "/", album_filename, ".pls", NULL);
+	g_free (album_filename);
 
 	gnome_vfs_unlink (filename);
 
@@ -367,23 +466,17 @@ save_playlist (DialogData *data)
 		for (scan = data->tracks; scan; scan = scan->next) {
 			TrackInfo *track = scan->data;
 			char      *track_filename;
-			/*char      *track_path;*/
 
 			n++;
 
-			track_filename = g_strdup_printf ("%s - %s.%s", zero_padded (track->number + 1), track->title, data->ext);
-
-			/*
-			track_path = g_build_filename (folder, track_filename, NULL);
-			*/
-
+			track_filename = get_track_filename (track, data->ext);
+			
 			sprintf (buffer, "File%d=%s\n", n, track_filename);
 			gnome_vfs_write (handle,
 					 buffer,
 					 strlen (buffer),
 					 NULL);
 			g_free (track_filename);
-			/*g_free (track_path);*/
 
 			sprintf (buffer, "Title%d=%s - %s\n", n, data->artist, track->title);
 			gnome_vfs_write (handle,
@@ -409,12 +502,13 @@ save_playlist (DialogData *data)
 static void
 rip_current_track (DialogData *data)
 {
-	TrackInfo *track;
-	char      *msg;
-	char      *filename;
-	char      *folder;
-	char      *e_title;
-	GstEvent  *event;
+	TrackInfo      *track;
+	char           *msg;
+	char           *filename;
+	char           *folder;
+	char           *e_title;
+	GstEvent       *event;
+	GnomeVFSResult  result;
 
 	if (data->current_track == NULL) {
 		GtkWidget *d;
@@ -458,9 +552,22 @@ rip_current_track (DialogData *data)
 	/* Set the filename */
 
 	folder = get_destination_folder (data);
-	ensure_dir_exists (folder, DESTINATION_PERMISSIONS);
+	
+	if ((result = ensure_dir_exists (folder, DESTINATION_PERMISSIONS)) != GNOME_VFS_OK) {
+		char *utf8_folder;
 
-	filename = g_strdup_printf ("%s - %s.%s", zero_padded (track->number + 1), track->title, data->ext);
+		utf8_folder = g_locale_to_utf8 (folder, -1, 0, 0, 0);
+		_gtk_error_dialog_run (GTK_WINDOW (data->window),
+				       "Could not extract tracks",
+				       "Could not create folder %s\n\n%s",
+				       utf8_folder,
+				       gnome_vfs_result_to_string (result));
+		g_free (utf8_folder);
+		gtk_widget_destroy (data->dialog);
+		return;
+	}
+
+	filename = get_track_filename (track, data->ext);
 	g_free (data->current_file);
 	data->current_file = g_build_filename (folder, filename, NULL);
 	g_free (filename);
@@ -475,7 +582,7 @@ rip_current_track (DialogData *data)
 
 	if (GST_IS_TAG_SETTER (data->encoder)) {
 		gst_tag_setter_add (GST_TAG_SETTER (data->encoder),   
-				    GST_TAG_MERGE_REPLACE_ALL,
+				    GST_TAG_MERGE_APPEND,
 				    GST_TAG_TITLE, track->title,
 				    GST_TAG_ARTIST, data->artist,
 				    GST_TAG_ALBUM, data->album,
@@ -529,17 +636,17 @@ start_ripper (DialogData *data)
 	goo_cdrom_lock_tray (data->cdrom);
 
 	data->prev_tracks_sectors = 0;
-
 	data->total_sectors = 0;
 	for (scan = data->tracks; scan; scan = scan->next) {
 		TrackInfo *track = scan->data;
 		data->total_sectors += track->sectors;
 	}
-
 	create_pipeline (data);
-
 	data->current_track_n = 1;
 	data->current_track = data->tracks;
+
+	g_timer_start (data->timer);
+
 	rip_current_track (data);
 }
 
@@ -584,7 +691,7 @@ dlg_ripper (GooWindow     *window,
 	data->tracks_n = g_list_length (data->tracks);
 	data->total_tracks = total_tracks;
 	data->update_handle = 0;
-
+	data->timer = g_timer_new ();
 	data->cdrom = goo_cdrom_new (location);
 
 	/* Get the widgets. */
